@@ -1,34 +1,13 @@
 {-# LANGUAGE TypeFamilies, GADTs, TupleSections #-}
-{-# OPTIONS_GHC -fno-warn-orphans #-}
 module Text.Regex.Applicative.Interface where
 import Control.Applicative hiding (empty)
-import qualified Control.Applicative
 import Control.Arrow
-import Data.Traversable
-import Data.String
+import Control.Monad (guard)
+import qualified Data.List as List
 import Data.Maybe
 import Text.Regex.Applicative.Types
 import Text.Regex.Applicative.Object
 
-instance Functor (RE s) where
-    fmap f x = Fmap f x
-    f <$ x = pure f <* x
-
-instance Applicative (RE s) where
-    pure x = const x <$> Eps
-    a1 <*> a2 = App a1 a2
-    a *> b = pure (const id) <*> Void a <*> b
-    a <* b = pure const <*> a <*> Void b
-
-instance Alternative (RE s) where
-    a1 <|> a2 = Alt a1 a2
-    empty = Fail
-    many a = reverse <$> Rep Greedy (flip (:)) [] a
-    some a = (:) <$> a <*> many a
-
-instance (char ~ Char, string ~ String) => IsString (RE char string) where
-    fromString = string
-
 -- | 'RE' is a profunctor. This is its contravariant map.
 --
 -- (A dependency on the @profunctors@ package doesn't seem justified.)
@@ -40,44 +19,15 @@ comap f re =
     Alt r1 r2     -> Alt (comap f r1) (comap f r2)
     App r1 r2     -> App (comap f r1) (comap f r2)
     Fmap g r      -> Fmap g (comap f r)
+    CatMaybes r   -> CatMaybes (comap f r)
     Fail          -> Fail
     Rep gr fn a r -> Rep gr fn a (comap f r)
     Void r        -> Void (comap f r)
 
--- | Match and return a single symbol which satisfies the predicate
-psym :: (s -> Bool) -> RE s s
-psym p = msym (\s -> if p s then Just s else Nothing)
-
--- | Like 'psym', but allows to return a computed value instead of the
--- original symbol
-msym :: (s -> Maybe a) -> RE s a
-msym p = Symbol (error "Not numbered symbol") p
-
--- | Match and return the given symbol
-sym :: Eq s => s -> RE s s
-sym s = psym (s ==)
-
 -- | Match and return any single symbol
 anySym :: RE s s
 anySym = msym Just
 
--- | Match and return the given sequence of symbols.
---
--- Note that there is an 'IsString' instance for regular expression, so
--- if you enable the @OverloadedStrings@ language extension, you can write
--- @string \"foo\"@ simply as @\"foo\"@.
---
--- Example:
---
--- >{-# LANGUAGE OverloadedStrings #-}
--- >import Text.Regex.Applicative
--- >
--- >number = "one" *> pure 1  <|>  "two" *> pure 2
--- >
--- >main = print $ "two" =~ number
-string :: Eq a => [a] -> RE a [a]
-string = traverse sym
-
 -- | Match zero or more instances of the given expression, which are combined using
 -- the given folding function.
 --
@@ -111,6 +61,8 @@ withMatched (App a b) =
         withMatched b
 withMatched Fail = Fail
 withMatched (Fmap f x) = (f *** id) <$> withMatched x
+withMatched (CatMaybes x) = CatMaybes $
+    (\ (as, s) -> flip (,) s <$> as) <$> withMatched x
 withMatched (Rep gr f a0 x) =
     Rep gr (\(a, s) (x, t) -> (f a x, s ++ t)) (a0, []) (withMatched x)
 -- N.B.: this ruins the Void optimization
@@ -147,6 +99,8 @@ match re = let obj = compile re in \str ->
 --
 -- If match is found, the rest of the input is also returned.
 --
+-- See also 'findFirstPrefixWithUncons', of which this is a special case.
+--
 -- Examples:
 --
 -- >Text.Regex.Applicative> findFirstPrefix ("a" <|> "ab") "abc"
@@ -156,24 +110,20 @@ match re = let obj = compile re in \str ->
 -- >Text.Regex.Applicative> findFirstPrefix "bc" "abc"
 -- >Nothing
 findFirstPrefix :: RE s a -> [s] -> Maybe (a, [s])
-findFirstPrefix re str = go (compile re) str Nothing
-    where
+findFirstPrefix = findFirstPrefixWithUncons List.uncons
+
+-- | Find the first prefix, with the given @uncons@ function.
+--
+-- @since 0.3.4
+findFirstPrefixWithUncons :: (ss -> Maybe (s, ss)) -> RE s a -> ss -> Maybe (a, ss)
+findFirstPrefixWithUncons = findPrefixWith' (walk emptyObject . threads)
+  where
     walk obj [] = (obj, Nothing)
     walk obj (t:ts) =
         case getResult t of
             Just r -> (obj, Just r)
             Nothing -> walk (addThread t obj) ts
 
-    go obj str resOld =
-        case walk emptyObject $ threads obj of
-            (obj', resThis) ->
-                let res = ((flip (,) str) <$> resThis) <|> resOld
-                in
-                    case str of
-                        _ | failed obj' -> res
-                        [] -> res
-                        (s:ss) -> go (step s obj') ss res
-
 -- | Find the longest string prefix which is matched by the regular expression.
 --
 -- Submatches are still determined using left bias and greediness, so this is
@@ -181,6 +131,8 @@ findFirstPrefix re str = go (compile re) str Nothing
 --
 -- If match is found, the rest of the input is also returned.
 --
+-- See also 'findLongestPrefixWithUncons', of which this is a special case.
+--
 -- Examples:
 --
 -- >Text.Regex.Applicative Data.Char> let keyword = "if"
@@ -191,28 +143,49 @@ findFirstPrefix re str = go (compile re) str Nothing
 -- >Text.Regex.Applicative Data.Char> findLongestPrefix lexeme "iffoo"
 -- >Just (Right "iffoo","")
 findLongestPrefix :: RE s a -> [s] -> Maybe (a, [s])
-findLongestPrefix re str = go (compile re) str Nothing
-    where
-    go obj str resOld =
-        let res = (fmap (flip (,) str) $ listToMaybe $ results obj) <|> resOld
-        in
-            case str of
-                _ | failed obj -> res
-                [] -> res
-                (s:ss) -> go (step s obj) ss res
+findLongestPrefix = findLongestPrefixWithUncons List.uncons
+
+-- | Find the longest prefix, with the given @uncons@ function.
+--
+-- @since 0.3.4
+findLongestPrefixWithUncons :: (ss -> Maybe (s, ss)) -> RE s a -> ss -> Maybe (a, ss)
+findLongestPrefixWithUncons = findPrefixWith' ((,) <*> listToMaybe . results)
+
+findPrefixWith'
+ :: (ReObject s a -> (ReObject s a, Maybe a))
+ -- ^ Given the regex object, compute the regex object to feed the next input value into, and
+ -- the result, if any.
+ -> (ss -> Maybe (s, ss)) -- ^ @uncons@
+ -> RE s a -> ss -> Maybe (a, ss)
+findPrefixWith' walk uncons = \ re -> go (compile re) Nothing
+  where
+    go obj resOld ss = case walk obj of
+        (obj', resThis) ->
+            let res = flip (,) ss <$> resThis <|> resOld
+            in
+                case uncons ss of
+                    _ | failed obj' -> res
+                    Nothing -> res
+                    Just (s, ss) -> go (step s obj') res ss
 
 -- | Find the shortest prefix (analogous to 'findLongestPrefix')
+--
+-- See also 'findShortestPrefixWithUncons', of which this is a special case.
 findShortestPrefix :: RE s a -> [s] -> Maybe (a, [s])
-findShortestPrefix re str = go (compile re) str
-    where
-    go obj str =
-        case results obj of
-            r : _ -> Just (r, str)
-            _ | failed obj -> Nothing
-            _ ->
-                case str of
-                    [] -> Nothing
-                    s:ss -> go (step s obj) ss
+findShortestPrefix = findShortestPrefixWithUncons List.uncons
+
+-- | Find the shortest prefix (analogous to 'findLongestPrefix'), with the given @uncons@ function.
+--
+-- @since 0.3.4
+findShortestPrefixWithUncons :: (ss -> Maybe (s, ss)) -> RE s a -> ss -> Maybe (a, ss)
+findShortestPrefixWithUncons uncons = go . compile
+  where
+    go obj ss = case results obj of
+        r:_ -> Just (r, ss)
+        _ -> do
+            guard (not (failed obj))
+            (s, ss) <- uncons ss
+            go (step s obj) ss
 
 -- | Find the leftmost substring that is matched by the regular expression.
 -- Otherwise behaves like 'findFirstPrefix'. Returns the result together with
